@@ -1,8 +1,8 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
-from trytond.model import ModelWorkflow, ModelView, ModelSQL, fields
-from trytond.pyson import Eval, If
+from trytond.model import Workflow, ModelView, ModelSQL, fields
+from trytond.pyson import Eval, If, Bool
 from trytond.transaction import Transaction
 from trytond.backend import TableHandler
 from trytond.pool import Pool
@@ -11,34 +11,35 @@ _STATES = {'readonly': Eval('state') != 'draft'}
 _DEPENDS = ['state']
 
 
-class Statement(ModelWorkflow, ModelSQL, ModelView):
+class Statement(Workflow, ModelSQL, ModelView):
     'Account Statement'
     _name = 'account.statement'
     _description = __doc__
 
     company = fields.Many2One('company.company', 'Company', required=True,
-        select=1, states=_STATES, domain=[
+        select=True, states=_STATES, domain=[
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', 0)),
             ],
         depends=_DEPENDS)
-    journal = fields.Many2One('account.statement.journal', 'Journal', required=True,
+    journal = fields.Many2One('account.statement.journal', 'Journal',
+        required=True,
         domain=[
             ('company', '=', Eval('context', {}).get('company', 0)),
             ],
         states={
             'readonly': (Eval('state') != 'draft') | Eval('lines'),
             },
-        on_change=['journal', 'state', 'lines'], select=1,
+        on_change=['journal', 'state', 'lines'], select=True,
         depends=['state', 'lines'])
     currency_digits = fields.Function(fields.Integer('Currency Digits',
         on_change_with=['journal']), 'get_currency_digits')
     date = fields.Date('Date', required=True, states=_STATES, depends=_DEPENDS,
-        select=1)
-    start_balance = fields.Numeric('Start Balance',
+        select=True)
+    start_balance = fields.Numeric('Start Balance', required=True,
         digits=(16, Eval('currency_digits', 2)),
         states=_STATES, depends=['state', 'currency_digits'])
-    end_balance = fields.Numeric('End Balance',
+    end_balance = fields.Numeric('End Balance', required=True,
         digits=(16, Eval('currency_digits', 2)),
         states=_STATES, depends=['state', 'currency_digits'])
     balance = fields.Function(
@@ -57,19 +58,38 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
         ('validated', 'Validated'),
         ('cancel', 'Canceled'),
         ('posted', 'Posted'),
-        ], 'State', readonly=True, select=1)
+        ], 'State', readonly=True, select=True)
     move_lines = fields.Function(fields.One2Many('account.move.line',
         None, 'Move Lines'), 'get_move_lines')
 
     def __init__(self):
         super(Statement, self).__init__()
-        self._rpc.update({
-            'draft_workflow': True,
-        })
         self._order[0] = ('id', 'DESC')
         self._error_messages.update({
-            'wrong_end_balance': 'End Balance must be %s!',
-            })
+                'wrong_end_balance': 'End Balance must be %s!',
+                'delete_cancel': 'Statement "%s" must be cancelled before ' \
+                    'deletion!',
+                })
+        self._transitions |= set((
+                ('draft', 'validated'),
+                ('validated', 'posted'),
+                ('validated', 'cancel'),
+                ('cancel', 'draft'),
+                ))
+        self._buttons.update({
+                'draft': {
+                    'invisible': Eval('state') != 'cancel',
+                    },
+                'validate': {
+                    'invisible': Eval('state') != 'draft',
+                    },
+                'post': {
+                    'invisible': Eval('state') != 'validated',
+                    },
+                'cancel': {
+                    'invisible': Eval('state') != 'validated',
+                    },
+                })
 
     def init(self, module_name):
         cursor = Transaction().cursor
@@ -96,7 +116,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
             table.not_null_action('company', action='add')
 
     def default_company(self):
-        return Transaction().context.get('company') or False
+        return Transaction().context.get('company')
 
     def default_state(self):
         return 'draft'
@@ -148,13 +168,10 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
         if not ids:
             return {}
 
-        for code in [Transaction().language, 'en_US']:
-            lang_ids = lang_obj.search([
-                ('code', '=', code),
+        lang_id, = lang_obj.search([
+                ('code', '=', Transaction().language),
                 ])
-            if lang_ids:
-                break
-        lang = lang_obj.browse(lang_ids[0])
+        lang = lang_obj.browse(lang_id)
 
         res = {}
         for statement in self.browse(ids):
@@ -169,12 +186,12 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
 
     def search_rec_name(self, name, clause):
         ids = self.search(['OR',
-            ('start_balance',) + tuple(clause[1:]),
-            ('end_balance',) + tuple(clause[1:]),
+            ('start_balance',) + clause[1:],
+            ('end_balance',) + clause[1:],
             ])
         if ids:
             return [('id', 'in', ids)]
-        return [('journal',) + tuple(clause[1:])]
+        return [('journal',) + clause[1:]]
 
     def get_move_lines(self, ids, name):
         '''
@@ -205,9 +222,10 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
 
     def on_change_with_balance(self, values):
         if not set(('end_balance', 'start_balance')) <= set(values):
-            return 0
+            return Decimal(0)
         else:
-            return values['end_balance'] - values['start_balance']
+            return Decimal(values.get('end_balance') or 0
+                - values.get('start_balance') or 0)
 
     def on_change_lines(self, values):
         pool = Pool()
@@ -226,9 +244,9 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
             invoice_id2amount_to_pay = {}
             for invoice in invoice_obj.browse(list(invoice_ids)):
                 with Transaction().set_context(date=invoice.currency_date):
-                    invoice_id2amount_to_pay[invoice.id] = currency_obj.compute(
-                        invoice.currency.id, invoice.amount_to_pay,
-                        journal.currency.id)
+                    invoice_id2amount_to_pay[invoice.id] = (
+                        currency_obj.compute(invoice.currency.id,
+                            invoice.amount_to_pay, journal.currency.id))
 
             for line in values['lines']:
                 if line['invoice'] and line['id']:
@@ -239,7 +257,7 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                                 amount_to_pay):
                             res['lines']['update'].append({
                                 'id': line['id'],
-                                'invoice': False,
+                                'invoice': None,
                                 })
                         else:
                             res['lines']['update'].append({
@@ -251,63 +269,72 @@ class Statement(ModelWorkflow, ModelSQL, ModelView):
                             res['lines'].setdefault('add', [])
                             vals = line.copy()
                             del vals['id']
-                            vals['amount'] = abs(line['amount']) - amount_to_pay
+                            vals['amount'] = (abs(line['amount'])
+                                - amount_to_pay)
                             if line['amount'] < 0:
                                 vals['amount'] = - vals['amount']
-                            vals['invoice'] = False
+                            vals['invoice'] = None
                             res['lines']['add'].append(vals)
                     invoice_id2amount_to_pay[line['invoice']] = \
                             amount_to_pay - abs(line['amount'])
         return res
 
-    def wkf_validated(self, statement):
+    def delete(self, ids):
+        if isinstance(ids, (int, long)):
+            ids = [ids]
+        # Cancel before delete
+        self.cancel(ids)
+        for statement in self.browse(ids):
+            if statement.state != 'cancel':
+                self.raise_user_error('delete_cancel', statement.rec_name)
+        return super(Statement, self).delete(ids)
+
+    @ModelView.button
+    @Workflow.transition('draft')
+    def draft(self, ids):
+        pass
+
+    @ModelView.button
+    @Workflow.transition('validated')
+    def validate(self, ids):
         statement_line_obj = Pool().get('account.statement.line')
         lang_obj = Pool().get('ir.lang')
 
-        computed_end_balance = statement.start_balance
-        for line in statement.lines:
-            computed_end_balance += line.amount
-        if computed_end_balance != statement.end_balance:
-            for code in [Transaction().language, 'en_US']:
-                lang_ids = lang_obj.search([
-                    ('code', '=', code),
-                    ])
-                if lang_ids:
-                    break
-            lang = lang_obj.browse(lang_ids[0])
+        for statement in self.browse(ids):
+            computed_end_balance = statement.start_balance
+            for line in statement.lines:
+                computed_end_balance += line.amount
+            if computed_end_balance != statement.end_balance:
+                lang_id, = lang_obj.search([
+                        ('code', '=', Transaction().language),
+                        ])
+                lang = lang_obj.browse(lang_id)
 
-            amount = lang_obj.format(lang,
-                    '%.' + str(statement.journal.currency.digits) + 'f',
-                    computed_end_balance, True)
-            self.raise_user_error('wrong_end_balance', error_args=(amount,))
-        for line in statement.lines:
-            statement_line_obj.create_move(line)
-        self.write(statement.id, {
-            'state':'validated',
-            })
+                amount = lang_obj.format(lang,
+                        '%.' + str(statement.journal.currency.digits) + 'f',
+                        computed_end_balance, True)
+                self.raise_user_error('wrong_end_balance',
+                    error_args=(amount,))
+            for line in statement.lines:
+                statement_line_obj.create_move(line)
 
-    def wkf_posted(self, statement):
+    @ModelView.button
+    @Workflow.transition('posted')
+    def post(self, ids):
         statement_line_obj = Pool().get('account.statement.line')
 
-        statement_line_obj.post_move(statement.lines)
-        self.write(statement.id, {
-            'state':'posted',
-            })
+        statements = self.browse(ids)
+        lines = [l for s in statements for l in s.lines]
+        statement_line_obj.post_move(lines)
 
-    def wkf_cancel(self, statement):
+    @ModelView.button
+    @Workflow.transition('cancel')
+    def cancel(self, ids):
         statement_line_obj = Pool().get('account.statement.line')
 
-        statement_line_obj.delete_move(statement.lines)
-        self.write(statement.id, {
-            'state':'cancel',
-            })
-
-    def draft_workflow(self, ids):
-        self.workflow_trigger_create(ids)
-        self.write(ids, {
-            'state': 'draft',
-            })
-        return True
+        statements = self.browse(ids)
+        lines = [l for s in statements for l in s.lines]
+        statement_line_obj.delete_move(lines)
 
 Statement()
 
@@ -321,24 +348,31 @@ class Line(ModelSQL, ModelView):
             required=True, ondelete='CASCADE')
     date = fields.Date('Date', required=True)
     amount = fields.Numeric('Amount', required=True,
-        digits=(16, Eval('_parent_statement', {}).get( 'currency_digits', 2)),
+        digits=(16, Eval('_parent_statement', {}).get('currency_digits', 2)),
         on_change=['amount', 'party', 'account', 'invoice',
             '_parent_statement.journal'])
     party = fields.Many2One('party.party', 'Party',
             on_change=['amount', 'party', 'invoice'])
     account = fields.Many2One('account.account', 'Account', required=True,
         on_change=['account', 'invoice'], domain=[
-            ('kind', '!=', 'view'),
             ('company', '=', Eval('_parent_statement', {}).get('company', 0)),
-            ])
+            If(Bool(Eval('party')),
+                If(Eval('amount', 0) > 0,
+                    ('kind', '=', 'receivable'),
+                    ('kind', '=', 'payable')),
+                ('kind', 'in', ['payable', 'receivable', 'revenue', 'expense',
+                        'other'])),
+            ],
+        depends=['party', 'amount'])
     description = fields.Char('Description')
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
     invoice = fields.Many2One('account.invoice', 'Invoice',
         domain=[
             ('party', '=', Eval('party')),
             ('account', '=', Eval('account')),
-            ('state', '=', If(Eval('_parent_statement', {}).get('state')
-                    == 'draft', 'open', '')),
+            If(Eval('_parent_statement', {}).get('state') == 'draft',
+                ('state', '=', 'open'),
+                ('state', '!=', '')),
             ],
         states={
             'readonly': ~Eval('amount'),
@@ -348,13 +382,20 @@ class Line(ModelSQL, ModelView):
     def __init__(self):
         super(Line, self).__init__()
         self._error_messages.update({
-            'debit_credit_account_statement_journal': 'Please provide debit and ' \
-                    'credit account on statement journal.',
-            'same_debit_credit_account': 'Credit or debit account on ' \
-                    'journal is the same than the statement line account!',
-            'amount_greater_invoice_amount_to_pay': 'Amount (%s) greater than '\
-                    'the amount to pay of invoice!',
+            'debit_credit_account_statement_journal': 'Please provide debit '
+                'and credit account on statement journal.',
+            'same_debit_credit_account': 'Credit or debit account on '
+                'journal is the same than the statement line account!',
+            'amount_greater_invoice_amount_to_pay': 'Amount (%s) greater than '
+                'the amount to pay of invoice!',
             })
+        self._sql_constraints += [
+            ('check_statement_line_amount', 'CHECK(amount != 0)',
+                'Amount should be a positive or negative value!'),
+            ]
+
+    def default_amount(self):
+        return Decimal(0)
 
     def on_change_party(self, value):
         party_obj = Pool().get('party.party')
@@ -375,9 +416,9 @@ class Line(ModelSQL, ModelView):
             if value.get('party'):
                 invoice = invoice_obj.browse(value['invoice'])
                 if invoice.party != value['party']:
-                    res['invoice'] = False
+                    res['invoice'] = None
             else:
-                res['invoice'] = False
+                res['invoice'] = None
         return res
 
     def on_change_amount(self, value):
@@ -404,14 +445,15 @@ class Line(ModelSQL, ModelView):
         if value.get('invoice'):
             if value.get('amount') and value.get('_parent_statement.journal'):
                 invoice = invoice_obj.browse(value['invoice'])
-                journal = journal_obj.browse(value['_parent_statement.journal'])
+                journal = journal_obj.browse(
+                    value['_parent_statement.journal'])
                 with Transaction().set_context(date=invoice.currency_date):
                     amount_to_pay = currency_obj.compute(invoice.currency.id,
                         invoice.amount_to_pay, journal.currency.id)
                 if abs(value['amount']) > amount_to_pay:
-                    res['invoice'] = False
+                    res['invoice'] = None
             else:
-                res['invoice'] = False
+                res['invoice'] = None
         return res
 
     def on_change_account(self, value):
@@ -422,17 +464,14 @@ class Line(ModelSQL, ModelView):
             if value.get('account'):
                 invoice = invoice_obj.browse(value['invoice'])
                 if invoice.account.id != value['account']:
-                    res['invoice'] = False
+                    res['invoice'] = None
             else:
-                res['invoice'] = False
+                res['invoice'] = None
         return res
 
     def create_move(self, line):
         '''
-        Create move for the statement line
-
-        :param line: a BrowseRecord of the line
-        :return: the move id
+        Create move for the statement line and return move id if created.
         '''
         pool = Pool()
         move_obj = pool.get('account.move')
@@ -442,12 +481,15 @@ class Line(ModelSQL, ModelView):
         move_line_obj = pool.get('account.move.line')
         lang_obj = pool.get('ir.lang')
 
+        if line.move:
+            return
+
         period_id = period_obj.find(line.statement.company.id,
                 date=line.date)
 
         move_lines = self._get_move_lines(line)
         move_id = move_obj.create({
-                'name': line.date,
+                'name': unicode(line.date),
                 'period': period_id,
                 'journal': line.statement.journal.journal.id,
                 'date': line.date,
@@ -464,17 +506,14 @@ class Line(ModelSQL, ModelView):
                         line.invoice.amount_to_pay,
                         line.statement.journal.currency.id)
             if amount_to_pay < abs(line.amount):
-                for code in [Transaction().language, 'en_US']:
-                    lang_ids = lang_obj.search([
-                        ('code', '=', code),
+                lang_id, = lang_obj.search([
+                        ('code', '=', Transaction().language),
                         ])
-                    if lang_ids:
-                        break
-                lang = lang_obj.browse(lang_ids[0])
+                lang = lang_obj.browse(lang_id)
 
                 amount = lang_obj.format(lang,
-                        '%.' + str(line.statement.journal.currency.digits) + 'f',
-                        line.amount, True)
+                    '%.' + str(line.statement.journal.currency.digits) + 'f',
+                    line.amount, True)
                 self.raise_user_error('amount_greater_invoice_amount_to_pay',
                         error_args=(amount,))
 
@@ -525,16 +564,17 @@ class Line(ModelSQL, ModelView):
             second_currency = statement_line.statement.journal.currency.id
             amount_second_currency = abs(statement_line.amount)
         else:
-            amount_second_currency = False
+            amount_second_currency = None
             second_currency = None
 
+        party_id = statement_line.party.id if statement_line.party else None
         vals = []
         vals.append({
-            'name': statement_line.date,
+            'name': unicode(statement_line.date),
             'debit': amount < zero and -amount or zero,
             'credit': amount >= zero and amount or zero,
             'account': statement_line.account.id,
-            'party': statement_line.party and statement_line.party.id,
+            'party': party_id,
             'second_currency': second_currency,
             'amount_second_currency': amount_second_currency,
             })
@@ -549,11 +589,11 @@ class Line(ModelSQL, ModelView):
         if statement_line.account.id == account.id:
             self.raise_user_error('same_debit_credit_account')
         vals.append({
-            'name': statement_line.date,
+            'name': unicode(statement_line.date),
             'debit': amount >= zero and amount or zero,
             'credit': amount < zero and -amount or zero,
             'account': account.id,
-            'party': statement_line.party and statement_line.party.id,
+            'party': party_id,
             'second_currency': second_currency,
             'amount_second_currency': amount_second_currency,
             })
